/**********************************************************************
 * Copyright (c) 2003, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TraceMethodBaseLoader.java,v 1.16 2009/05/05 15:51:13 paules Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: TraceMethodBaseLoader.java,v 1.16 2009/05/05 15:51:13 paules Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.hyades.loaders.internal.binary.v1.BinaryTraceConstants;
import org.eclipse.hyades.loaders.trace.TraceUtils.InvocationInfo;
import org.eclipse.hyades.loaders.trace.TraceUtils.InvocationPool;
import org.eclipse.hyades.loaders.util.AgentsContext;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.hierarchy.CorrelationSourceInfo;
import org.eclipse.hyades.models.hierarchy.HierarchyFactory;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.UnresolvedCorrelation;
import org.eclipse.hyades.models.hierarchy.impl.UnresolvedCorrelationImpl;
import org.eclipse.hyades.models.hierarchy.util.FastList;
import org.eclipse.hyades.models.hierarchy.util.HierarchyResourceSetImpl;
import org.eclipse.hyades.models.hierarchy.util.LongToObjectMap;
import org.eclipse.hyades.models.hierarchy.util.SaveUtil;
import org.eclipse.hyades.models.trace.TRCClass;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCFullTraceObject;
import org.eclipse.hyades.models.trace.TRCInputOutputContainer;
import org.eclipse.hyades.models.trace.TRCMethod;
import org.eclipse.hyades.models.trace.TRCMethodInvocation;
import org.eclipse.hyades.models.trace.TRCObject;
import org.eclipse.hyades.models.trace.TRCObjectValue;
import org.eclipse.hyades.models.trace.TRCObjectValueKind;
import org.eclipse.hyades.models.trace.TRCObjectValuePosition;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.models.trace.TRCTypedObjectValue;
import org.eclipse.hyades.models.trace.TraceFactory;
import org.eclipse.hyades.models.trace.TracePackage;
import org.eclipse.hyades.models.trace.impl.TRCFullMethodInvocationImpl;
import org.eclipse.hyades.models.trace.impl.TRCThreadImpl;
import org.eclipse.hyades.models.util.ModelDebugger;

/**
 * @author slavescu
 */
public class TraceMethodBaseLoader extends TraceXMLFragmentLoader {
	// ~ Static fields/initializers
	// -----------------------------------------------------------------
	protected static final String TICKET = "ticket";
	protected static final String STACK_DEPTH = "stackDepth";
	protected static final String PARAMETER = "parameter";
	protected static final String RETURN_VALUE = "returnValue";
	protected static final String VALUE = "value";
	protected static final String VALUE_KIND = "valueKind";
	protected static final String VALUE_POSITION = "valuePos";
	protected static final String PARAM_INDEX = "index";

	protected static final String PARAMETER_ID_REF = "parameterIdRef";

	protected static final String INVOCATION_CONTEXT = "InvocationContext";
	protected static final String AGENT_ID_REF = "agentIdRef";
	protected static final String PROCESS_ID_REF = "processIdRef";
	protected static final String NODE_ID_REF = "nodeIdRef";
	protected static final String SEQUENCE_COUNTER = "sequenceCounter";

	protected static final String REMOTE_AGENT_ID_REF = "remoteAgentIdRef";
	protected static final String REMOTE_PROCESS_ID_REF = "remoteProcessIdRef";
	protected static final String REMOTE_NODE_ID_REF = "remoteNodeIdRef";
	protected static final String REMOTE_SEQUENCE_COUNTER = "remoteSequenceCounter";
	protected static final String REMOTE_THREAD_ID_REF = "remoteThreadIdRef";
	protected static final String REMOTE_TICKET = "remoteTicket";

	// ~ Instance fields
	// ----------------------------------------------------------------------------
	protected CallStackPerThread _cs;
	protected InvocationPool _invocationPool;
	protected TRCClass _invokerClass;
	protected TRCClass _invokerObjectClass;
	protected TRCFullMethodInvocation _fullInvocation;
	protected TRCFullMethodInvocation _fullInvoker;
	protected TRCFullTraceObject _invokerObject;
	
	/** invoker here seems to refer to the method being processed (entered/exited)*/
	protected TRCMethod _invokerMethod;
	protected double _deltaBaseTime;
	protected double _deltaCumulativeTime;
	protected double _lastChildExitTime;
	protected double _previousSiblingExitTime;
	protected long _ticket;
	protected short _stackDepth;
	protected List<TRCObjectValue> _inputValues = new FastList<TRCObjectValue>();
	protected List<TRCObjectValue> _outputValues = new FastList<TRCObjectValue>();
	protected InvocationContext _invocationContext = new InvocationContext();
	protected boolean _activeInvocationContext;
	protected long _sequenceCounter;
	protected static final Object _REMOTE_INVOCATION_RESOLUTION_LOCK = new Object();
	protected boolean _remoteInvocationContext;
	// protected static final String USE_updateStatisticalInfoOld = "USE_updateStatisticalInfoOld";
	
	protected int _childId;
	protected boolean _useUpdateStatisticalInfoOld = false;

	// ~ Methods
	// ------------------------------------------------------------------------------------
	protected void updateTimeStatisticsNew() {
		updateBaseTime();

		// update cumulative time
		// current invocation should be on the stack for methodExit but not for
		// methodEntry
		for (int i = _cs.size(); i-- > 0;) {
			
			InvocationInfo invocationInfo = (InvocationInfo) _cs.get(i);

			if (!invocationInfo.isObjectLocked()) {
				TRCFullTraceObject obj = invocationInfo.getObject();
				if (obj != null)
					obj.incrementCumulativeTime(_deltaCumulativeTime);
//					obj.setCumulativeTime(obj.getCumulativeTime() + _deltaCumulativeTime);
			}

			// Not used for JVMTI
//			TRCClass objectClass = invocationInfo.getObjectClass();
//			if ((objectClass != null) && !invocationInfo.isObjectClassLocked()) {
//				objectClass.setInheritedCumulativeTime(objectClass.getInheritedCumulativeTime() + _deltaCumulativeTime);
//				// if(!invocationInfo.isPackageLocked())
//				objectClass.getPackage().setInheritedCumulativeTime(objectClass.getPackage().getInheritedCumulativeTime() + _deltaCumulativeTime);
//				objectClass.getPackage().getProcess().setInheritedCumulativeTime(objectClass.getPackage().getProcess().getInheritedCumulativeTime() + _deltaCumulativeTime);
//			}
			
			if (!invocationInfo.isMethodLocked()) {
				invocationInfo.getMethod().incrementCumulativeTime(_deltaCumulativeTime);
//				invocationInfo.getMethod().setCumulativeTime(invocationInfo.getMethod().getCumulativeTime() + _deltaCumulativeTime);
			}
			
			if (!invocationInfo.isClassLocked()) {
				_invokerClass = invocationInfo.getTheClass();
				_invokerClass.incrementCumulativeTime(_deltaCumulativeTime);
//				_invokerClass.setCumulativeTime(_invokerClass.getCumulativeTime() + _deltaCumulativeTime);
				
				if (!invocationInfo.isPackageLocked()) {
					_invokerClass.getPackage().incrementCumulativeTime(_deltaCumulativeTime);
//					_invokerClass.getPackage().setCumulativeTime(_invokerClass.getPackage().getCumulativeTime() + _deltaCumulativeTime);
				}
	
				_invokerClass.getPackage().getProcess().incrementCumulativeTime(_deltaCumulativeTime);
//				_invokerClass.getPackage().getProcess().setCumulativeTime(_invokerClass.getPackage().getProcess().getCumulativeTime() + _deltaCumulativeTime);
			}
		}
	}

	// ~ Methods
	// ------------------------------------------------------------------------------------
	protected void updateBaseTime() {
		// update base time
//		_invokerMethod.setBaseTime(_invokerMethod.getBaseTime() + _deltaBaseTime);
		_invokerMethod.incrementBaseTime(_deltaBaseTime);
		
		if (_invokerObject != null) {
//			_invokerObject.setBaseTime(_invokerObject.getBaseTime() + _deltaBaseTime);
			_invokerObject.incrementBaseTime(_deltaBaseTime);
			
//			if (_invokerObjectClass != null) {
//				_invokerObjectClass.setInheritedBaseTime(_invokerObjectClass.getInheritedBaseTime() + _deltaBaseTime);
//				_invokerObjectClass.getPackage().setInheritedBaseTime(_invokerObjectClass.getPackage().getInheritedBaseTime() + _deltaBaseTime);
//				_invokerObjectClass.getPackage().getProcess().setInheritedBaseTime(_invokerObjectClass.getPackage().getProcess().getInheritedBaseTime() + _deltaBaseTime);
//			}
		}
		
		_invokerClass.incrementBaseTime(_deltaBaseTime);
//		_invokerClass.setBaseTime(_invokerClass.getBaseTime() + _deltaBaseTime);
		
		_invokerClass.getPackage().incrementBaseTime(_deltaBaseTime);
//		_invokerClass.getPackage().setBaseTime(_invokerClass.getPackage().getBaseTime() + _deltaBaseTime);
		
		_invokerClass.getPackage().getProcess().incrementBaseTime(_deltaBaseTime);
//		_invokerClass.getPackage().getProcess().setBaseTime(_invokerClass.getPackage().getProcess().getBaseTime() + _deltaBaseTime);
	}

	protected void updateTimeStatistics() {
		updateBaseTime();
		
		// update cumulative time
		// current invocation should be on the stack for methodExit but not for
		// methodEntry
		// 
		// Update the cumulative time of all things that are higher up on the thread stack, as their
		// cumulative time should grow by the base time of the current method 
		for (int i = _cs.size(); i-- > 0;) {
			InvocationInfo invocationInfo = (InvocationInfo) _cs.get(i);
			if (!invocationInfo.isObjectLocked()) {
				TRCFullTraceObject obj = invocationInfo.getObject();
				if (obj != null)
					obj.incrementCumulativeTime(_deltaBaseTime);
//					obj.setCumulativeTime(obj.getCumulativeTime() + deltaBaseTime);
			}
			
//			TRCClass objectClass = invocationInfo.getObjectClass();
//			if ((objectClass != null) && !invocationInfo.isObjectClassLocked()) {
//				objectClass.setInheritedCumulativeTime(objectClass.getInheritedCumulativeTime() + _deltaBaseTime);
//				// if(!invocationInfo.isPackageLocked())
//				objectClass.getPackage().setInheritedCumulativeTime(objectClass.getPackage().getInheritedCumulativeTime() + _deltaBaseTime);
//				objectClass.getPackage().getProcess().setInheritedCumulativeTime(objectClass.getPackage().getProcess().getInheritedCumulativeTime() + _deltaBaseTime);
//			}
			
			if (!invocationInfo.isMethodLocked()) {
//				invocationInfo.getMethod().setCumulativeTime(invocationInfo.getMethod().getCumulativeTime() + deltaBaseTime);
				invocationInfo.getMethod().incrementCumulativeTime(_deltaBaseTime);
			}
			
			if (!invocationInfo.isClassLocked()) {
				_invokerClass = invocationInfo.getTheClass();
//				invokerClass.setCumulativeTime(invokerClass.getCumulativeTime() + deltaBaseTime);
				_invokerClass.incrementCumulativeTime(_deltaBaseTime);
				if(!invocationInfo.isPackageLocked()) {
//					invokerClass.getPackage().setCumulativeTime(invokerClass.getPackage().getCumulativeTime() + deltaBaseTime);
					_invokerClass.getPackage().incrementCumulativeTime(_deltaBaseTime);
				}
				
//				invokerClass.getPackage().getProcess().setCumulativeTime(invokerClass.getPackage().getProcess().getCumulativeTime() + deltaBaseTime);
				_invokerClass.getPackage().getProcess().incrementCumulativeTime(_deltaBaseTime);
			}
		}
	}
	protected TRCClass getExtendedClass(TRCObject anObject, TRCClass aClass) {
		if (anObject != null) {
			if (anObject.getIsA() != aClass) {
				if (anObject.getIsA() != theProcess.getClassClass())
					return anObject.getIsA();
			}
		}
		return null;
	}

	/**
	 * @param theThread
	 */
	protected void setMaxStackDepth(TRCThread theThread) {
		if (theThread == null)
			return;
		if (_stackDepth > theThread.getMaxStackDepth())
			theThread.setMaxStackDepth(_stackDepth);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.hyades.loaders.hierarchy.IgnoredXMLFragmentLoader#startChild(java.lang.String)
	 */
	public void startChild(String name) {
		super.startChild(name);
		if (name.equals(PARAMETER)) {
			objectValue = TraceFactory.eINSTANCE.createTRCObjectValue();
			_inputValues.add(objectValue);
			_childId = TraceConstants.PARAMETER_int;
		} else if (name.equals(RETURN_VALUE)) {
			objectValue = TraceFactory.eINSTANCE.createTRCObjectValue();
			_outputValues.add(objectValue);
			_childId = TraceConstants.RETURN_VALUE_int;
		} else {
			_childId = 0;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.hyades.loaders.hierarchy.IgnoredXMLFragmentLoader#endChild(java.lang.String)
	 */
	public void endChild(String name) {
		super.endChild(name);
		if (name.equals(PARAMETER) || name.equals(RETURN_VALUE)) {
			objectValue = null;
			_childId = 0;
		}
	}

	public void addInputOutputValues() {
		if (_inputValues.isEmpty() && _outputValues.isEmpty())
			return;
		EList<TRCObjectValue> l = (EList<TRCObjectValue>) getInputOutputContainer().getEntries().get(_fullInvocation);
		if (l == null) {
			l = new BasicEList<TRCObjectValue>(0);
			getInputOutputContainer().getEntries().put(_fullInvocation, l);
			l = (EList<TRCObjectValue>) getInputOutputContainer().getEntries().get(_fullInvocation);
		}
		if (!_outputValues.isEmpty()) {
			// getInputOutputContainer().getInputOutputValues().add(outputValues.get(outputValues.size() - 1));
			if (l.isEmpty())
				l.add(_outputValues.get(_outputValues.size() - 1));
			else
				l.set(0, _outputValues.get(_outputValues.size() - 1)); // last value wins
		} else if (l.isEmpty()) {
			TRCObjectValue objectValue = TraceFactory.eINSTANCE.createTRCObjectValue();
			// getInputOutputContainer().getInputOutputValues().add(objectValue);
			l.add(objectValue); // reserve the first entry for return value
		}
		if (!_inputValues.isEmpty()) {
			// int ls1 = l.size()-1;
			// int ls2 = inputValues.size();
			// int gapSize = Math.abs(ls1-ls2);
			// if(ls1>0 && ls1<ls2)
			// {
			// //fill gap
			// for (int i = 0; i < gapSize; i++) {
			// l.add(TraceFactory.eINSTANCE.createTRCObjectValue());
			// }
			// }
			l.addAll(_inputValues); // concatenate all input values at the end of the list
			// if(ls1>0 && ls2<ls1)
			// {
			// //fill gap
			// for (int i = 0; i < gapSize; i++) {
			// l.add(TraceFactory.eINSTANCE.createTRCObjectValue());
			// }
			// }
		}
	}

	/**
	 * @return
	 */
	protected TRCInputOutputContainer getInputOutputContainer() {
		TRCInputOutputContainer inputOutputContainer = getProcess().getInputOutputContainer();
		if (inputOutputContainer == null) {
			inputOutputContainer = TraceFactory.eINSTANCE.createTRCInputOutputContainer();
			inputOutputContainer.setProcess(getProcess());
			Resource r = createInputOutputContainerResource(inputOutputContainer);
			r.getContents().add(inputOutputContainer);
		}
		return inputOutputContainer;
	}

	/**
	 * @param inputOutputContainer
	 */
	protected Resource createInputOutputContainerResource(TRCInputOutputContainer inputOutputContainer) {
		String uri = inputOutputContainer.getProcess().eResource().getURI().toString();
		String newUri = uri.substring(0, uri.indexOf(".trca"));
		newUri = newUri.concat(".trciov").concat(uri.substring(uri.indexOf(".trca") + 5));
		return HierarchyResourceSetImpl.getInstance().createResource(SaveUtil.createURI(newUri));
	}

	
	@Override
	public void addAttribute(int con, byte value) {
		// No byte attrs		
	}

	@Override
	public void addAttribute(int con, short value) {
		switch(con) {
		case BinaryTraceConstants.STACK_DEPTH:
			_stackDepth  = value;
			break;
		default:
			super.addAttribute(con, value);
		}
	}

	@Override
	public void addAttribute(int con, int value) {
		switch(con) {
		default:
			// THREAD_ID_REF, METHOD_ID_REF			
			super.addAttribute(con, value); 
			break;
		}
		
	}

	@Override
	public void addAttribute(int con, long value) {
		
		switch(con) {
		
		case BinaryTraceConstants.TICKET:
			if (!_activeInvocationContext) {
				_ticket = value;
			} else {
				_invocationContext.setInvocationTicket(value);
			}
			break;
			
		case BinaryTraceConstants.TRANS_THREAD_ID_REF:
		case BinaryTraceConstants.TRANS_OBJ_ID_REF:
		case BinaryTraceConstants.THREAD_CPU_TIME:
		case BinaryTraceConstants.SEQUENCE_COUNTER:
		case BinaryTraceConstants.TRANS_CLASS_ID_REF: 		// method exit
		// ignored
		break;
		
		default:
			// CLASS_ID_REF, OBJ_ID_REF
			super.addAttribute(con, value); 
			break;
		}		

	}

	@Override
	public void addAttribute(int con, double value) {
		// No double attrs
	}

	@Override
	public void addAttribute(int con, String value) {
		switch(con) {
		case BinaryTraceConstants.COLLATION_VALUE:
		case BinaryTraceConstants.TRACE_ID_REF:
			// ignored
			break;
		default:
			// TIME
			super.addAttribute(con, value); 
			break;
		}
				
	}

	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.hyades.loaders.trace.TraceXMLFragmentLoader#addAttribute(java.lang.String, java.lang.String)
	 */
	public void addAttribute(String name, String value) {
		switch (LoadersUtils.getHashCode(name)) {
		case TraceConstants.STACK_DEPTH_int:
			_stackDepth = Short.parseShort(value);
			break;
		case TraceConstants.TICKET_int:
			if (!_activeInvocationContext) {
				_ticket = Long.parseLong(value);
			} else {
				_invocationContext.setInvocationTicket(Long.parseLong(value));
			}
			break;
		case TraceConstants.SEQUENCE_COUNTER_int:
			if (!_activeInvocationContext) {
				_sequenceCounter = Long.parseLong(value);
			} else {
				_invocationContext.setInvocationSequenceCounter(Long.parseLong(value));
			}
			break;
		case TraceConstants.NODE_ID_REF_int:
			if (_activeInvocationContext) {
				_invocationContext.setInvocationNodeIdRef(value);
			}
			break;
		case TraceConstants.PROCESS_ID_REF_int:
			if (_activeInvocationContext) {
				_invocationContext.setInvocationProcessIdRef(value);
			}
			break;
		case TraceConstants.AGENT_ID_REF_int:
			if (_activeInvocationContext) {
				_invocationContext.setInvocationAgentIdRef(value);
			}
			break;
		case TraceConstants.THREAD_ID_REF_int:
			if (!_activeInvocationContext) {
				threadIdRef = Integer.parseInt(value);
			} else {
				_invocationContext.setInvocationThreadIdRef(Integer.parseInt(value));
			}
			break;
		case TraceConstants.REMOTE_AGENT_ID_REF_int:
			if (!_remoteInvocationContext) {
				setNullInvocationContext();
				_remoteInvocationContext = true;
			}
			_invocationContext.setInvocationAgentIdRef(value);
			break;
		case TraceConstants.REMOTE_NODE_ID_REF_int:
			if (!_remoteInvocationContext) {
				setNullInvocationContext();
				_remoteInvocationContext = true;
			}
			_invocationContext.setInvocationNodeIdRef(value);
			break;
		case TraceConstants.REMOTE_PROCESS_ID_REF_int:
			if (!_remoteInvocationContext) {
				setNullInvocationContext();
				_remoteInvocationContext = true;
			}
			_invocationContext.setInvocationProcessIdRef(value);
			break;
		case TraceConstants.REMOTE_THREAD_ID_REF_int:
			if (!_remoteInvocationContext) {
				setNullInvocationContext();
				_remoteInvocationContext = true;
			}
			_invocationContext.setInvocationThreadIdRef(Integer.parseInt(value));
			break;
		case TraceConstants.REMOTE_SEQUENCE_COUNTER_int:
			if (!_remoteInvocationContext) {
				setNullInvocationContext();
				_remoteInvocationContext = true;
			}
			_invocationContext.setInvocationSequenceCounter(Long.parseLong(value));
			break;
		case TraceConstants.REMOTE_TICKET_int:
			if (!_remoteInvocationContext) {
				setNullInvocationContext();
				_remoteInvocationContext = true;
			}
			_invocationContext.setInvocationTicket(Long.parseLong(value));
			break;
		case TraceConstants.PARAMETER_ID_REF_int:
			// do nothing
			break;
		case TraceConstants.VALUE_KIND_int:
			if (objectValue != null) {
				convertToTypedObjectValue();
				((TRCTypedObjectValue) objectValue).setValueKind(TRCObjectValueKind.get(value));
			}
			break;
		case TraceConstants.VALUE_POSITION_int:
			if (objectValue != null) {
				convertToTypedObjectValue();
				((TRCTypedObjectValue) objectValue).setPositionKind(TRCObjectValuePosition.get(value));
			}
			break;
		case TraceConstants.PARAM_INDEX_int:
			if (objectValue != null) {
				convertToTypedObjectValue();
				((TRCTypedObjectValue) objectValue).setPosition(Short.parseShort(value));
			}
			break;

		default:
			super.addAttribute(name, value);
			break;
		}
	}

	protected void convertToTypedObjectValue() {
		if (objectValue instanceof TRCTypedObjectValue)
			return;

		TRCTypedObjectValue newObjectValue = TraceFactory.eINSTANCE.createTRCTypedObjectValue();
		newObjectValue.setStringValue(objectValue.getStringValue());

		if (_childId == TraceConstants.PARAMETER_int) {
			int i = _inputValues.indexOf(objectValue);

			if (i < 0)
				_inputValues.add(newObjectValue);
			else
				_inputValues.set(i, newObjectValue);
		} else if (_childId == TraceConstants.RETURN_VALUE_int) {
			int i = _outputValues.indexOf(objectValue);

			if (i < 0)
				_outputValues.add(newObjectValue);
			else
				_outputValues.set(i, newObjectValue);
		}
		objectValue = newObjectValue;
	}

	protected void setNullInvocationContext() {
		_invocationContext.setInvocationAgentIdRef(null);
		_invocationContext.setInvocationNodeIdRef(null);
		_invocationContext.setInvocationProcessIdRef(null);
		_invocationContext.setInvocationSequenceCounter(0);
		_invocationContext.setInvocationThreadIdRef(0);
		_invocationContext.setInvocationTicket(0);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.hyades.loaders.trace.TraceXMLFragmentLoader#initialize(org.eclipse.hyades.loaders.util.HierarchyContext, java.lang.String)
	 */
	public void initialize(HierarchyContext context, String name) {
		super.initialize(context, name);
		_activeInvocationContext = false;
		_remoteInvocationContext = false;
		_stackDepth = 0;
		_ticket = 0;
		_sequenceCounter = 0;
		_inputValues.clear();
		_outputValues.clear();
		objectValue = null;
		
		_invocationPool = null;
		_invokerClass = null;
		_invokerObjectClass = null;
		_fullInvocation = null;
		_fullInvoker = null;
		_invokerObject = null;
		
	}

	protected TRCFullMethodInvocation resolveInvocation(InvocationContext invContext) {
		TRCFullMethodInvocation targetInvocation = null;
		try {
			TRCAgent targetAgent = null;
			// HierarchyContext invokerContext;
			AgentsContext agentsContext = (AgentsContext) LookupServiceExtensions.getInstance().locate(null, AgentsContext.class,
					LoadersUtils.getLookUpKey(invContext.getInvocationAgentIdRef()));
			if (agentsContext != null) {
				targetAgent = agentsContext.getActiveAgent(context.getAgent());
			}
			if (targetAgent != null) {
				HierarchyContext targetContext = LoadersUtils.locateHierarchyContext(targetAgent);
				if (targetContext != null) {
					// targetInvocation = (TRCFullMethodInvocation) LookupServiceExtensions.getInstance().locate(targetContext, TRCFullMethodInvocation.class,
					// LoadersUtils.getHashCode(TraceUtils.getMethodInvocationId(invContext)));
					LongToObjectMap ticketsMap = (LongToObjectMap) LookupServiceExtensions.getInstance().locate(targetContext, TRCFullMethodInvocation.class,
							invContext.getInvocationThreadIdRef());
					if (ticketsMap != null) {
						targetInvocation = (TRCFullMethodInvocation) ticketsMap.get(invContext.getInvocationTicket());
						if (targetInvocation == null) {
							boolean inOrder = false;
							TRCThread targetThread = (TRCThread) LookupServiceExtensions.getInstance().locate(targetContext, TRCThreadImpl.class,
									invContext.getInvocationThreadIdRef());
							if (targetThread != null) {
								Iterator<TRCMethodInvocation> invocationIterator = targetThread.getInitialInvocations().iterator();
								while (invocationIterator.hasNext()) {
									_fullInvoker = (TRCFullMethodInvocation) invocationIterator.next();
									if (!TRCFullMethodInvocationImpl.class.isAssignableFrom(_fullInvoker.getClass())) {
										continue;
									}
									if ((_fullInvoker.getThread() == targetThread) && (((TRCFullMethodInvocationImpl) _fullInvoker).getTicket() == invContext.getInvocationTicket())) {
										inOrder = true;
										break;
									}
									if (checkTicket(targetThread, _fullInvoker.getInvokes(), invContext.getInvocationTicket())) {
										inOrder = true;
										break;
									}
								}
							}
							if (inOrder) {
								targetInvocation = _fullInvoker;
							}
						}
					}
				}
			}
		} catch (Exception e) {
			ModelDebugger.log(e);
		}
		return targetInvocation;
	}

	protected boolean checkTicket(TRCThread theThread, EList<TRCMethodInvocation> list, long ticket) {
		boolean inOrder = false;
		for (Iterator<TRCMethodInvocation> iter = list.iterator(); iter.hasNext();) {
			_fullInvoker = (TRCFullMethodInvocation) iter.next();
			if (_fullInvoker.getClass() != TRCFullMethodInvocationImpl.class) {
				continue;
			}
			if ((_fullInvoker.getThread() == theThread) && (((TRCFullMethodInvocationImpl) _fullInvoker).getTicket() == ticket)) {
				inOrder = true;
				break;
			}
			if (checkTicket(theThread, _fullInvoker.getInvokes(), ticket)) {
				inOrder = true;
				break;
			}
		}
		return inOrder;
	}

	/**
	 * @return The invoker's TRCMethodInvocation.
	 */
	protected TRCMethodInvocation setInvokerMethod(TRCThread thread, TRCMethodInvocation invocation) {
		/*
		 * If there is no invocation context then we were invoked on the current
		 * thread and we just need to look at the top of the stack to find our
		 * parent. Otherwise, we need to to look for our invoker based upon the
		 * context information we have. If we cannot find our invoker we can add
		 * this to the bottom of the stack initially and then update it later
		 * when our invoker is loaded.
		 */
		if (_activeInvocationContext || _remoteInvocationContext) {
			_fullInvoker = resolveInvocation(_invocationContext);
			if (_fullInvoker == null) {
				/*
				 * If we get this far we have arrived before our invokers method entry, so add us to the callee of the remote method
				 */
				addForwardInvocation(invocation);
			} else {
				// System.out.println("Remote Invocation -- In order");
				invocation.setInvokedBy(_fullInvoker);
				// updateBaseTimeIfRequired(invocation,fullInvoker);
			}
		} else {
			/* Local invocation or we could not find our invoker */
			if ((_cs != null) && !_cs.isEmpty()) {
				_fullInvoker = ((InvocationInfo) _cs.peek()).getMethodInvocation();
			}
			if (_fullInvoker != null) {
				if (_fullInvoker.getStackDepth() < _stackDepth) {
					invocation.setInvokedBy(_fullInvoker);
					// updateBaseTimeIfRequired(invocation,fullInvoker);
				} else {
					while (_fullInvoker.getInvokedBy() != null && _fullInvoker.getInvokedBy() instanceof TRCFullMethodInvocation) {
						_fullInvoker = (TRCFullMethodInvocation) _fullInvoker.getInvokedBy();
						if (_fullInvoker.getStackDepth() < _stackDepth) {
							invocation.setInvokedBy(_fullInvoker);
							// updateBaseTimeIfRequired(invocation,fullInvoker);
							break;
						}
					}
				}
			}
		}
		return _fullInvoker;
	}

	protected void addForwardInvocation(TRCMethodInvocation invocation) {
		String contextId = _invocationContext.getInvocationAgentIdRef() + "/" + _invocationContext.getInvocationThreadIdRef() + "/" + _invocationContext.getInvocationTicket() + "/"
				+ _invocationContext.getInvocationSequenceCounter();
		UnresolvedCorrelation unresolvedCorrelation = HierarchyFactory.eINSTANCE.createUnresolvedCorrelation();
		unresolvedCorrelation.setContextId(contextId);
		CorrelationSourceInfo correlationSourceInfo = HierarchyFactory.eINSTANCE.createCorrelationSourceInfo();
		correlationSourceInfo.setOwner(invocation);
		correlationSourceInfo.setReference(TracePackage.eINSTANCE.getTRCMethodInvocation_InvokedBy());
		unresolvedCorrelation.getSourceInfos().add(correlationSourceInfo);
		unresolvedCorrelation.setAgent(context.getAgent());
	}

	/**
	 * Resolves the links to posible target invocations, invocation that were received before the invoker (sourceInvocation)
	 */
	protected void updateForwardInvokes(TRCMethodInvocation sourceInvocation) {
		// InvocationContext source;
		UnresolvedCorrelation unresolvedCorrelation = (UnresolvedCorrelation) LookupServiceExtensions.getInstance().locate(null, UnresolvedCorrelationImpl.class, /*
																																								 * context.getNode().
																																								 * getRuntimeId() +
																																								 * "/" + context.
																																								 * getProcessProxy
																																								 * ().getRuntimeId()
																																								 * + "/" +
																																								 */
				context.getAgent().getRuntimeId() + "/" + threadIdRef + "/" + _ticket + "/" + _sequenceCounter);
		
		if (unresolvedCorrelation != null) {
			for (Iterator<CorrelationSourceInfo> iterator = unresolvedCorrelation.getSourceInfos().iterator(); iterator.hasNext();) {
				CorrelationSourceInfo correlationSourceInfo = iterator.next();
				EReference reference = correlationSourceInfo.getReference();
				EObject owner = correlationSourceInfo.getOwner();
				if (owner instanceof TRCMethodInvocation && (reference == TracePackage.eINSTANCE.getTRCMethodInvocation_InvokedBy())) {
					if (sourceInvocation != null) {
						((TRCMethodInvocation) owner).setInvokedBy(sourceInvocation);
						// //adjust base time
						// updateBaseTimeIfRequired((TRCMethodInvocation)owner, sourceInvocation);
						iterator.remove();
					}
				}
			}
			if (unresolvedCorrelation.getSourceInfos().size() == 0) {
				unresolvedCorrelation.setAgent(null);
			}
		}
	}

	protected void updateBaseTimeIfRequired(TRCMethodInvocation callee, TRCMethodInvocation caller) {
		if (callee instanceof TRCFullMethodInvocation && caller instanceof TRCFullMethodInvocation) {
			TRCFullMethodInvocation cr = (TRCFullMethodInvocation) caller;
			TRCFullMethodInvocation ce = (TRCFullMethodInvocation) callee;
			if (cr.eIsSet(TracePackage.eINSTANCE.getTRCFullMethodInvocation_ExitTime()) && ce.eIsSet(TracePackage.eINSTANCE.getTRCFullMethodInvocation_ExitTime())) {
				_deltaBaseTime = -(ce.getExitTime() - ce.getEntryTime() - ce.getOverhead());
				_deltaCumulativeTime = 0;

				_invokerMethod = cr.getMethod();
				_invokerObject = (TRCFullTraceObject) cr.getOwningObject();
				_invokerClass = cr.getMethod().getDefiningClass();

				_invokerObjectClass = getExtendedClass(_invokerObject, _invokerClass);
				updateBaseTime();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.hyades.loaders.hierarchy.IgnoredXMLFragmentLoader#addCharacters(char[], int, int)
	 */
	public void addCharacters(char[] data, int offset, int length) {
		if (objectValue != null) {
			String s = (new String(data, offset, length));
			if (s.trim().length() > 0)
				objectValue.setStringValue(objectValue.getStringValue() + s);
		}

	}
}