/*******************************************************************************
 * Copyright (c) 2000, 2016 IBM Corporation, Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.trace.ui.internal.wizard;

import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.internal.core.TraceProfileFiltersUI;
import org.eclipse.hyades.trace.ui.internal.util.FilterSetElement;
import org.eclipse.hyades.trace.ui.internal.util.TraceMessages;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;



public class ExportProfileSettingWizard extends Wizard implements IImportWizard{
	
	private IStructuredSelection selection;
	private ExportProfileSettingOptionPage optionPage;
	private ExportProfileSettingWizardPage selectionPage;
	protected TraceProfileFiltersUI filterUI;
	protected FilterSetElement FSelement;
	private static final String WIZARD_SETTINGS_KEY = "FileExportWizard"; //$NON-NLS-1$
	
	@Override
	public boolean performFinish() {
			return selectionPage.performFinish();
		}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#canFinish()
	 */
	public boolean canFinish() {
		return selectionPage.isPageComplete();
		}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 *      org.eclipse.jface.viewers.IStructuredSelection)
	 */
	
	public void init(IWorkbench workbench, IStructuredSelection sel, TraceProfileFiltersUI filterUI, FilterSetElement element){
		this.filterUI = filterUI;
		this.FSelement = element;
		init(workbench, sel);
	}
	public void init(IWorkbench workbench, IStructuredSelection sel) {
		setNeedsProgressMonitor(true);
		setWindowTitle(TraceMessages.EXPORT_WIZ_SUP_TITLE);
		setHelpAvailable(false);
		this.selection = sel;

		IDialogSettings pluginSettings = UIPlugin.getDefault()
				.getDialogSettings();
		IDialogSettings wizardSettings = pluginSettings
				.getSection(WIZARD_SETTINGS_KEY);

		if (wizardSettings == null) {
			wizardSettings = pluginSettings.addNewSection(WIZARD_SETTINGS_KEY);
		}
		setDialogSettings(pluginSettings);
		
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		optionPage = new ExportProfileSettingOptionPage(selection);
		selectionPage = new ExportProfileSettingWizardPage(selection);
		selectionPage.setFilterUI(this.filterUI);
		selectionPage.setFSelement(this.FSelement);

		addPage(optionPage);
		addPage(selectionPage);
	}
	
	@Override
	public IWizardPage getNextPage(IWizardPage page) {
		if (page instanceof ExportProfileSettingOptionPage) {
			selectionPage.setExportType(optionPage.getSelectedExportType());
			return selectionPage;
		}
		return null;
	}
}
