/*******************************************************************************
 * Copyright (c) 2000, 2016 IBM Corporation, Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.trace.ui.internal.wizard;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.internal.core.TraceFilterManager;
import org.eclipse.hyades.trace.ui.internal.core.TraceProfileFiltersUI;
import org.eclipse.hyades.trace.ui.internal.util.FilterSetElement;
import org.eclipse.hyades.trace.ui.internal.util.FilterTableElement;
import org.eclipse.hyades.trace.ui.internal.util.TraceMessages;
import org.eclipse.hyades.ui.internal.util.XMLUtil;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.WizardResourceImportPage;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.w3c.dom.Element;


public class ImportProfileSettingWizardPage extends WizardResourceImportPage{
	
	protected Text ProfileFilterSourceField;
	protected Button ProfileFilterSourceButton;
	protected int ImportType =0;
	public int getImportType() {
		return ImportType;
	}

	public void setImportType(int importType) {
		ImportType = importType;
	}
	protected static String[] EXTs = {".xml", ".txt"};
	protected TraceProfileFiltersUI filterUI;
	
	public TraceProfileFiltersUI getFilterUI() {
		return filterUI;
	}

	public void setFilterUI(TraceProfileFiltersUI filterUI) {
		this.filterUI = filterUI;
	}

	public ImportProfileSettingWizardPage(IStructuredSelection sel) {
		super(TraceMessages.IMPORT_WIZ_SUP_TITLE, sel);
		setTitle(TraceMessages.IMPORT_WIZ_TITLE);
		setDescription(TraceMessages.IMPORT_WIZ_DES);
		//this.selection = sel;
	}
	
	private Control[] createTextFieldWithBrowse(Composite parent,
			String labelName) {

		/* Create a new composite */
		Composite sourceContainerGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		sourceContainerGroup.setLayout(layout);
		sourceContainerGroup.setFont(parent.getFont());
		sourceContainerGroup.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		/* Create the label */
		Label groupLabel = new Label(sourceContainerGroup, SWT.NONE);
		groupLabel.setText(labelName);
		groupLabel.setFont(parent.getFont());

		/* Create the Text object */
		Text sourceNameField = new Text(sourceContainerGroup, SWT.BORDER);
		GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.GRAB_HORIZONTAL);
		sourceNameField.setLayoutData(data);
		sourceNameField.setFont(parent.getFont());

		sourceNameField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				checkPageValid();

			}
		});

		/* Create the Browse button */
		Button sourceBrowseButton = new Button(sourceContainerGroup, SWT.PUSH);
		sourceBrowseButton
				.setText(TraceMessages.BROWSE2);
		sourceBrowseButton.addListener(SWT.Selection, this);
		GridData g = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		sourceBrowseButton.setLayoutData(g);
		sourceBrowseButton.setFont(parent.getFont());
		setButtonLayoutData(sourceBrowseButton);

		return new Control[] { sourceNameField, sourceBrowseButton };
	}
	
	@SuppressWarnings("restriction")
	@Override
	public void createControl(Composite composite) {

		/* Create a main composite */
		Composite coverageContainer = new Composite(composite, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		coverageContainer.setLayout(layout);
		coverageContainer.setFont(composite.getFont());
		coverageContainer.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL
						| GridData.FILL_BOTH));

		/* Create the search boxes for the coveragedata files */
		Control[] widgets = createTextFieldWithBrowse(coverageContainer,
				TraceMessages.IMPORT_FILTER_NAME);
		ProfileFilterSourceField = (Text) widgets[0];
		ProfileFilterSourceButton = (Button) widgets[1];
		ProfileFilterSourceField.setToolTipText(TraceMessages.IMPORT_FILTER_SOURCE_HELP);
		ProfileFilterSourceButton.setToolTipText(TraceMessages.IMPORT_FILTER_SOURCE_BRO_HELP);

		setControl(coverageContainer);
		

		setPageComplete(false);
		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, UIPlugin.getPluginId() + ".pfpsi0002");
	}
	
	@Override
	public void handleEvent(Event event) {

		if (event.widget == ProfileFilterSourceButton) {
			/* Handle the 'Browse' button on the coverage data textbox */
			handleSourceBrowseButtonPressed();
		} 
	}
	
	protected void handleSourceBrowseButtonPressed() {
		FileDialog dialog = new FileDialog(ProfileFilterSourceField.getShell(),
				SWT.OPEN);
		String dataFileExtension = EXTs[ImportType]; //$NON-NLS-1$

		StringBuffer filterName = new StringBuffer();
		filterName.append(dataFileExtension);
		
		dialog.setFilterNames(new String[] {filterName.toString()});
		dialog.setFilterExtensions(
				new String[] {"*" + dataFileExtension});

		String selectedFile = dialog.open();
		if (selectedFile != null) {
			// If it is valid then proceed to populate
			setErrorMessage(null);
			ProfileFilterSourceField.setText(selectedFile);
			ProfileFilterSourceField.setFocus();
		}
	}
	
	protected boolean ensureSourceIsValid() {
		if (new File(ProfileFilterSourceField.getText()).isFile()) {
			return true;
		}
		return false;
	}
	
	private boolean checkPageValid() {
		if (!ensureSourceIsValid()) {
			setErrorMessage(TraceMessages.IMPORT_SOURCE_ERR);
		} else {
			setErrorMessage(null);
			setPageComplete(true);
			return true;
		}
		setPageComplete(false);
		return false;
	}
	
	public boolean performFinish() {
		try {
			if (!importResource())
				return false;
		} catch (Exception e) {
			return false;
		}
		return true;
	}
	private boolean importResource() throws CoreException,
	InvocationTargetException, InterruptedException {
	
		
		File FilterFileToImport = getLaunchFileToImport();
		
		if (ImportType == ImportProfileSettingOptionPage.XMLTYPE){
			importXMLFilterSetList(FilterFileToImport);
		}
		else if(ImportType == ImportProfileSettingOptionPage.TXTTYPE){
			showNewPlainImportDialog(FilterFileToImport);
		}
		 
		return true;
		}
	
	private File getLaunchFileToImport(){
		return new File(ProfileFilterSourceField.getText());
	}
	
	public void importXMLFilterSetList(File FilterFileToImport){
		TraceFilterManager tfm = TraceFilterManager.getInstance();  
		try {
			BufferedReader inputReader = new BufferedReader(new InputStreamReader(new FileInputStream(FilterFileToImport), "UTF8")); //$NON-NLS-1$
			StringBuilder result = new StringBuilder();
			String nextLine = inputReader.readLine();
			while (nextLine != null){
				result.append(nextLine);
				nextLine = inputReader.readLine();
			}
			inputReader.close();
			
			String stringresult = result.toString();
			Element fitlerElements = XMLUtil.loadDom(stringresult, "filters");
			ArrayList<FilterSetElement> list = tfm.parseFiltersSet(fitlerElements);
			for (FilterSetElement ele: list){
				if (!verifyID(ele.getId())){
					ele.setId(ele.getId() + System.currentTimeMillis());
				}
				filterUI.HandleEditDialogOption(TraceMessages.IMPORT_RENAME_DIA_TITLE, ele,  ele.getDisplayName());
		}		
		//tfm.storeFilterSetList(oldList);
		return;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return;
	}
	
	private boolean verifyID (String id){
		for (int idx = 0; idx < filterUI.getfilterSetList().getSize(); idx++) {
			FilterSetElement el = (FilterSetElement) filterUI.getfilterSetList().getElementAt(idx);
			if (el.getId().equals(id)) {
				return false;
			}
		}
		return true;
	}
	private boolean verifyName(String content){
		for (int idx = 0; idx < filterUI.getfilterSetList().getSize(); idx++) {
			FilterSetElement el = (FilterSetElement) filterUI.getfilterSetList().getElementAt(idx);
			if (el.getDisplayName().equals(content)) {
				return false;
			}
	}
		return true;
	}
	
public void showNewPlainImportDialog(File FilterFileToImport){
		
		
				FilterSetElement elem =	new FilterSetElement(UIPlugin.PLUGIN_ID + System.currentTimeMillis());
				elem.setKey(null);
				elem.setName(TraceMessages.IMPORT_PLAINTXT_DEFAULT +  " [ " + DateFormat.getDateTimeInstance().format(new Date()) + " ]");
				elem.setChildren(new ArrayList());
				
					//TODO
					BufferedReader inputReader;
					try {
						inputReader = new BufferedReader(new InputStreamReader(new FileInputStream(FilterFileToImport), "UTF8")); //$NON-NLS-1$
					String nextLine = inputReader.readLine();
					while (nextLine != null){
						nextLine = nextLine.replaceAll(" {1,}", ",");
						String[] collection= nextLine.split(",");
						//require at least 3 attributes to construct a filter 
						if (collection == null || collection.length < 3)
							return;
						FilterTableElement temp = new FilterTableElement(collection[0], collection[1], collection[2]);
						elem.getChildren().add(temp);
						nextLine = inputReader.readLine();
					}
					inputReader.close();
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					filterUI.showWSetDialog(UIPlugin.getResourceString(TraceMessages.IMPORT_RENAME_DIA_TITLE), elem);
				
			}

@Override
protected void createSourceGroup(Composite arg0) {
	// TODO Auto-generated method stub
	
}

@Override
protected ITreeContentProvider getFileProvider() {
	// TODO Auto-generated method stub
	return new WorkbenchContentProvider();
}

@Override
protected ITreeContentProvider getFolderProvider() {
	// TODO Auto-generated method stub
	return new WorkbenchContentProvider();
}
	
}
