/**********************************************************************
 Copyright (c) 2007, 2014 IBM Corporation and others.
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License v1.0
 which accompanies this distribution, and is available at
 http://www.eclipse.org/legal/epl-v10.html
 $Id: ConfigGenerator.java,
 
 Contributors:
     IBM - initial implementation
 **********************************************************************/
package org.eclipse.tptp.platform.iac.administrator;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.preferences.DefaultScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.tptp.platform.iac.administrator.internal.common.CommonConstants;
import org.eclipse.tptp.platform.iac.administrator.internal.common.Trace;
import org.eclipse.tptp.platform.iac.administrator.internal.preference.VMChangeListener;
import org.eclipse.tptp.platform.iac.administrator.internal.startstop.AutoStartStop;
import org.osgi.framework.BundleContext;
import org.osgi.service.prefs.BackingStoreException;

/**
 * The activator class controls the plug-in life cycle
 * 
 * @author Navid Mehregani
 */
public class AdminPlugin extends Plugin {

	/* The plug-in ID */
	public static final String PLUGIN_ID = "org.eclipse.tptp.platform.iac.administrator";

	/* The shared instance */
	private static AdminPlugin plugin;

	private static IEclipsePreferences instancePreferences;
	private static IEclipsePreferences defaultPreferences;

	/**
	 * The constructor
	 */
	public AdminPlugin() {
		super();
	}

	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;

		instancePreferences = InstanceScope.INSTANCE
				.getNode(AdminPlugin.PLUGIN_ID);
		defaultPreferences = DefaultScope.INSTANCE
				.getNode(AdminPlugin.PLUGIN_ID);

		Trace.initialize(context, PLUGIN_ID);

		JavaRuntime.addVMInstallChangedListener(new VMChangeListener());
	}

	public void stop(BundleContext context) throws Exception {
		/*
		 * NM: Bugzilla 173330: Terminate the Agent Controller if it has been
		 * started
		 */
		AutoStartStop.stopIAC();

		// JPT: Moved this after the stopIAC call to prevent NPE in error
		// handling code of AutoStartStop.stopIAC()
		instancePreferences = null;
		defaultPreferences = null;
		plugin = null;

		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static AdminPlugin getDefault() {
		return plugin;
	}

	/* The following methods are used to manipulate the preference store */
	public boolean getDefaultBoolean(String name) {

		boolean result = false;
		if (defaultPreferences != null) {
			result = defaultPreferences.getBoolean(name, false);
		}
		return result;
	}

	public int getDefaultInt(String name) {

		int result = 0;
		if (defaultPreferences != null) {
			result = defaultPreferences.getInt(name, 0);
		}
		return result;
	}

	public String getDefaultString(String name) {

		String result = CommonConstants.EMPTY_STRING;
		if (defaultPreferences != null) {
			result = defaultPreferences.get(name, CommonConstants.EMPTY_STRING);
		}
		return result;
	}

	public boolean getBoolean(String name) {

		boolean result = false;
		if (instancePreferences != null) {
			result = instancePreferences.getBoolean(name,
					this.getDefaultBoolean(name));
		}
		return result;
	}

	public int getInt(String name) {

		int result = 0;
		if (instancePreferences != null) {
			result = instancePreferences.getInt(name, this.getDefaultInt(name));
		}
		return result;
	}

	public String getString(String name) {

		String result = CommonConstants.EMPTY_STRING;
		if (instancePreferences != null) {
			result = instancePreferences.get(name, this.getDefaultString(name));
		}
		return result;
	}

	public void setBoolean(String name, boolean value) {

		if ((instancePreferences != null) && (name != null)) {
			instancePreferences.putBoolean(name, value);
		}
		this.flushPreferences();
	}

	public void setInt(String name, int value) {

		if ((instancePreferences != null) && (name != null)) {
			instancePreferences.putInt(name, value);
		}
		this.flushPreferences();
	}

	public void setString(String name, String value) {
		String n = name;
		String v = value;

		if (n == null) {
			n = CommonConstants.EMPTY_STRING;
		}
		if (v == null) {
			v = CommonConstants.EMPTY_STRING;
		}

		if ((instancePreferences != null)) {
			instancePreferences.put(name, value);
		}
		this.flushPreferences();
	}

	public void flushPreferences() {

		try {
			instancePreferences.flush();
		} catch (BackingStoreException backingStoreEx) {
			AdminPlugin
					.getDefault()
					.getLog()
					.log(new Status(IStatus.ERROR, AdminPlugin.PLUGIN_ID,
							backingStoreEx.getMessage(), backingStoreEx));
		}
	}
}