/**********************************************************************
 * Copyright (c) 2007, 2014 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: IACPreferencePageInitializer.java,v 1.1 2007-04-12 18:02:53 nmehrega Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.tptp.platform.iac.administrator.internal.preference;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.core.runtime.preferences.DefaultScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.tptp.platform.iac.administrator.AdminPlugin;
import org.eclipse.tptp.platform.iac.administrator.internal.common.AdminUtil;
import org.eclipse.tptp.platform.iac.administrator.internal.common.CommonConstants;
import org.osgi.service.prefs.BackingStoreException;

/**
 * This class is used initialize the values on Agent Controller's preference
 * page
 * 
 * @author Navid Mehregani
 *
 */
public class IACPreferencePageInitializer extends AbstractPreferenceInitializer {

	/**
	 * Set up the default values for the Agent Controller preference page
	 */
	public void initializeDefaultPreferences() {

		IEclipsePreferences node = DefaultScope.INSTANCE
				.getNode(AdminPlugin.PLUGIN_ID);
		node.putBoolean(PreferenceMessages.ENABLE_DISABLE_LABEL, true);
		node.put(PreferenceMessages.JAVA_EXECUTABLE,
				AdminUtil.getJVMExecutable());
		node.put(PreferenceMessages.ALLOWED_HOSTS, PreferenceMessages.LOCALHOST);
		node.putBoolean(PreferenceMessages.AC_SECURITY, false);
		node.put(PreferenceMessages.ALLOWED_USERS, PreferenceMessages.ANY_USER);
		node.putBoolean(CommonConstants.DATAPOOL_PORT_ENABLED, false);
		node.putBoolean(CommonConstants.DATAPOOL_PORT_RANDOM, true);
		node.put(CommonConstants.DATAPOOL_PORT_NUMBER,
				CommonConstants.EMPTY_STRING);
		try {
			node.flush();
		} catch (BackingStoreException e) {
			IStatus status = new Status(IStatus.ERROR, AdminPlugin.PLUGIN_ID,
					IStatus.OK, e.getMessage(), e);
			AdminPlugin.getDefault().getLog().log(status);
		}
	}
}