#!/bin/bash
#
# ACStart.sh - Starts the Agent Controller server process.
#

#
# If TPTP_AC_HOME is not set then set it to the parent directory of the directory containing this script
#
if [ -z "$TPTP_AC_HOME" ]; then
	TPTP_AC_HOME=`dirname "$0"`/..
	export TPTP_AC_HOME
fi
PATH="$TPTP_AC_HOME/bin":$PATH
export PATH

#
# Make sure TPTP_AC_HOME is valid
#
if [ -e "$TPTP_AC_HOME/bin/ACServer" ] ; then
	echo "Starting Agent Controller."
else
	echo "TPTP_AC_HOME not set properly. Could not locate bin/ACServer at that location."
	echo "TPTP_AC_HOME=" $TPTP_AC_HOME
	exit 67
fi

#
# Bug 294802, check for the serviceconfig.xml
# Parse the configuration file for the location of the jvm library
#
if [ -e "$TPTP_AC_HOME/config/serviceconfig.xml" ] ; then
    JVM_LIB=`grep '<Jvm' "$TPTP_AC_HOME/config/serviceconfig.xml" | sed 's/<Jvm location="\(.*\)"\/>/\1/'`
    export JVM_LIB

#
# Set the library path(s) based on the location of the jvm library (".." is the parent directory which
# also holds other dependent *.so)
#
    LD_LIBRARY_PATH="$TPTP_AC_HOME/lib":`echo $JVM_LIB | sed 's/\(.*\)\/libjvm\.so/\1/'`:`echo $JVM_LIB | sed 's/\(.*\)\/libjvm\.so/\1/' | sed 's/\(.*\)\/.*/\1/'`:/usr/lib:$LD_LIBRARY_PATH
else
    LD_LIBRARY_PATH="$TPTP_AC_HOME/lib":/usr/lib:$LD_LIBRARY_PATH
fi
export LD_LIBRARY_PATH

#
# Bug 175696, 201989
# Make sure TEMP dir is set
#
if [[ -n $TEMP ]]; then
	if [[ -d $TEMP ]]; then
		echo
	else
		export TEMP=/tmp
	fi
else
	export TEMP=/tmp
fi

#
# Start the ACServer
#
RUNNING=`ps -ef | grep "ACServer" | grep -v "grep" | wc -l`
if [ $RUNNING -ne 0 ];then
	ps -ef | grep "ACServer" | grep -v "grep" | awk '{
	 	print "The server is already running on pid " $2
	}'      
	exit 66
else
	# Increase the stack size for this process, if possible (redirect error output to null)
	ulimit -s unlimited &> /dev/null
    ACServer "$@" &
fi

#
# Wait for the process to come up in the system process list
#
sleep 2

#
# Check if ACServer is running by querying the system process list
#
RUNNING=`ps -ef | grep "ACServer" | grep -v "grep" | wc -l`
if [ $RUNNING -ne 0 ];then
    echo "ACServer started successfully."
    exit 0
else
    echo "ACServer failed to start."
    exit 1
fi
