/*******************************************************************************
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2015 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

#ifndef __BCI_TRACE_H_
#define __BCI_TRACE_H_

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#ifdef HPUX
#	include <iostream.h>
#	include <fstream.h>
#else
#	include <iostream>
#	include <fstream>
#endif

#ifdef MVS
#    include <unistd.h> /* for __etoa */
#endif 

#if defined(MVS) || defined(__OS400__)
#	include <strstream>
#endif

#if defined(_SOLARIS) || defined(_SOLARISX86)
#define __FUNCTION__ __FILE__
#endif

class BCITrace
{
public:
	static BCITrace& instance();
	void traceBCI(const char* fn,
			   unsigned line,
			   const char* fmt,
			   va_list va);

	void traceJClass(const char* fn,
			   unsigned line,
			   const char* fmt,
			   va_list va);
	
	void trace(const char* groupName,
		       const char* fn,
			   unsigned line,
			   const char* fmt,
			   va_list va);

	bool isTraceEnabled()
	{
		if (_file)
			return true;
		return false;
	}

private:
	BCITrace();
	~BCITrace();

	FILE* _file;
};

inline void traceBCI(const char* fn, unsigned line, const char* fmt, ...)
{
	va_list va;
	va_start(va, fmt);
	BCITrace::instance().traceBCI(fn, line, fmt, va);
}

inline void traceJClass(const char* fn, unsigned line, const char* fmt, ...)
{
	va_list va;
	va_start(va, fmt);
	BCITrace::instance().traceJClass(fn, line, fmt, va);
}

inline bool isTraceEnabled()
{
	return BCITrace::instance().isTraceEnabled();
}

#endif // __BCI_TRACE_H_
